@@ -7,6 +7,99 @@ from . import http
 from . import error
 
 
+class ChoiceData:
+    """
+    Command choice data object
+
+    :ivar name: Name of the choice, this is what the user will see
+    :ivar value: Values of the choice, this is what discord will return to you
+    """
+
+    def __init__(self, name, value):
+        self.name = name
+        self.value = value
+
+    def __eq__(self, other):
+        return isinstance(other, ChoiceData) and self.__dict__ == other.__dict__
+
+
+class OptionData:
+    """
+    Command option data object
+
+    :ivar name: Name of the option.
+    :ivar description: Description of the option.
+    :ivar required: If the option is required.
+    :ivar choices: A list of :class:`ChoiceData`, cannot be present on subcommand groups
+    :ivar options: List of :class:`OptionData`, this will be present if it's a subcommand group
+    """
+
+    def __init__(
+            self, name, description, required=False, choices=None, options=None, **kwargs
+    ):
+        self.name = name
+        self.description = description
+        self.type = kwargs.get("type")
+        if self.type is None:
+            raise error.IncorrectCommandData("type is required for options")
+        self.required = required
+        if choices is not None:
+            self.choices = []
+            for choice in choices:
+                self.choices.append(ChoiceData(**choice))
+        else:
+            self.choices = None
+
+        if self.type in (1, 2):
+            self.options = []
+            if options is not None:
+                for option in options:
+                    self.options.append(OptionData(**option))
+            elif self.type == 2:
+                raise error.IncorrectCommandData(
+                    "Options are required for subcommands / subcommand groups"
+                )
+
+    def __eq__(self, other):
+        return isinstance(other, OptionData) and self.__dict__ == other.__dict__
+
+
+class CommandData:
+    """
+    Slash command data object
+
+    :ivar name: Name of the command.
+    :ivar description: Description of the command.
+    :ivar options: List of :class:`OptionData`.
+    :ivar id: Command id, this is received from discord so may not be present
+    """
+
+    def __init__(
+            self, name, description, options=None, id=None, application_id=None, version=None, **kwargs
+    ):
+        self.name = name
+        self.description = description
+        self.id = id
+        self.application_id = application_id
+        self.version = version
+        if options is not None:
+            self.options = []
+            for option in options:
+                self.options.append(OptionData(**option))
+        else:
+            self.options = None
+
+    def __eq__(self, other):
+        if isinstance(other, CommandData):
+            return (
+                    self.name == other.name
+                    and self.description == other.description
+                    and self.options == other.options
+            )
+        else:
+            return False
+
+
 class CommandObject:
     """
     Slash command object of this extension.
@@ -22,6 +115,7 @@ class CommandObject:
     :ivar connector: Kwargs connector of the command.
     :ivar __commands_checks__: Check of the command.
     """
+
     def __init__(self, name, cmd):  # Let's reuse old command formatting.
         self.name = name.lower()
         self.func = cmd["func"]
@@ -97,8 +191,9 @@ class SubcommandObject(CommandObject):
     :ivar base_description: Description of the base command.
     :ivar subcommand_group_description: Description of the subcommand_group.
     """
+
     def __init__(self, sub, base, name, sub_group=None):
-        sub["has_subcommands"] = True # For the inherited class.
+        sub["has_subcommands"] = True  # For the inherited class.
         super().__init__(name, sub)
         self.base = base.lower()
         self.subcommand_group = sub_group.lower() if sub_group else sub_group
@@ -113,6 +208,7 @@ class CogCommandObject(CommandObject):
     .. warning::
         Do not manually init this model.
     """
+
     def __init__(self, *args):
         super().__init__(*args)
         self.cog = None  # Manually set this later.
@@ -138,6 +234,7 @@ class CogSubcommandObject(SubcommandObject):
     .. warning::
         Do not manually init this model.
     """
+
     def __init__(self, *args):
         super().__init__(*args)
         self.cog = None  # Manually set this later.
@@ -188,45 +285,64 @@ class SlashCommandOptionType(IntEnum):
 
 class SlashMessage(discord.Message):
     """discord.py's :class:`discord.Message` but overridden ``edit`` and ``delete`` to work for slash command."""
+
     def __init__(self, *, state, channel, data, _http: http.SlashCommandRequest, interaction_token):
         # Yes I know it isn't the best way but this makes implementation simple.
         super().__init__(state=state, channel=channel, data=data)
         self._http = _http
         self.__interaction_token = interaction_token
 
+    async def _slash_edit(self, **fields):
+        """
+        An internal function
+        """
+        _resp = {}
+
+        content = fields.get("content")
+        if content:
+            _resp["content"] = str(content)
+
+        embed = fields.get("embed")
+        embeds = fields.get("embeds")
+        file = fields.get("file")
+        files = fields.get("files")
+
+        if embed and embeds:
+            raise error.IncorrectFormat("You can't use both `embed` and `embeds`!")
+        if file and files:
+            raise error.IncorrectFormat("You can't use both `file` and `files`!")
+        if file:
+            files = [file]
+        if embed:
+            embeds = [embed]
+        if embeds:
+            if not isinstance(embeds, list):
+                raise error.IncorrectFormat("Provide a list of embeds.")
+            elif len(embeds) > 10:
+                raise error.IncorrectFormat("Do not provide more than 10 embeds.")
+            _resp["embeds"] = [x.to_dict() for x in embeds]
+
+        allowed_mentions = fields.get("allowed_mentions")
+        _resp["allowed_mentions"] = allowed_mentions.to_dict() if allowed_mentions else \
+            self._state.allowed_mentions.to_dict() if self._state.allowed_mentions else {}
+
+        await self._http.edit(_resp, self.__interaction_token, self.id, files=files)
+
+        delete_after = fields.get("delete_after")
+        if delete_after:
+            await self.delete(delay=delete_after)
+        if files:
+            [x.close() for x in files]
+
     async def edit(self, **fields):
         """Refer :meth:`discord.Message.edit`."""
-        try:
-            await super().edit(**fields)
-        except discord.Forbidden:
-            _resp = {}
-
-            content = str(fields.get("content"))
-            if content:
-                _resp["content"] = str(content)
-
-            embed = fields.get("embed")
-            embeds = fields.get("embeds")
-            if embed and embeds:
-                raise error.IncorrectFormat("You can't use both `embed` and `embeds`!")
-            if embed:
-                embeds = [embed]
-            if embeds:
-                if not isinstance(embeds, list):
-                    raise error.IncorrectFormat("Provide a list of embeds.")
-                elif len(embeds) > 10:
-                    raise error.IncorrectFormat("Do not provide more than 10 embeds.")
-                _resp["embeds"] = [x.to_dict() for x in embeds]
-
-            allowed_mentions = fields.get("allowed_mentions")
-            _resp["allowed_mentions"] = allowed_mentions.to_dict() if allowed_mentions else \
-                self._state.allowed_mentions.to_dict() if self._state.allowed_mentions else {}
-
-            await self._http.edit(_resp, self.__interaction_token, self.id)
-
-            delete_after = fields.get("delete_after")
-            if delete_after:
-                await self.delete(delay=delete_after)
+        if "file" in fields or "files" in fields:
+            await self._slash_edit(**fields)
+        else:
+            try:
+                await super().edit(**fields)
+            except discord.Forbidden:
+                await self._slash_edit(**fields)
 
     async def delete(self, *, delay=None):
         """Refer :meth:`discord.Message.delete`."""
@@ -240,4 +356,5 @@ class SlashMessage(discord.Message):
                 with suppress(discord.HTTPException):
                     await asyncio.sleep(delay)
                     await self._http.delete(self.__interaction_token, self.id)
+
             self._state.loop.create_task(wrap())
